/*
 * Copyright (c) 1993 Addison-Wesley Publishing Company.
 * All rights reserved.  This source code is provided
 * for its instructional value, and is not guaranteed
 * for any particular purpose.
 */
#include <sys/types.h>
#include <tiuser.h>
#include <fcntl.h>
#include <stdio.h>
#include <memory.h>
#include <stdlib.h>

extern int t_errno;
extern char *t_errlist[];

extern void log(const char *, ...);

#define tlog(STR) log("%s: %s", (STR), t_errlist[t_errno])

int
initserver(char *transport, char *addr, uint_t alen,
	uint_t qlen)
{
	int fd;
	struct t_bind *reqp, *retp;

	/*
	 * Open the transport provider.
	 */
	if ((fd = t_open(transport, O_RDWR, NULL)) < 0) {
		tlog("Cannot open network");
		exit(1);
	}

	/*
	 * Allocate t_bind structures for the requested and
	 * returned addresses.
	 */
	reqp = (struct t_bind *)t_alloc(fd, T_BIND, T_ADDR);
	retp = (struct t_bind *)t_alloc(fd, T_BIND, T_ADDR);
	if ((reqp == NULL) || (retp == NULL)) {
		tlog("Cannot allocate bind structures");
		exit(1);
	}

	/*
	 * Copy the requested address to the t_bind structure
	 * and bind the address to the transport endpoint.
	 */
	memcpy(reqp->addr.buf, addr, alen);
	reqp->addr.len = alen;
	reqp->qlen = qlen;
	if (t_bind(fd, reqp, retp) < 0) {
		tlog("Cannot bind address");
		exit(1);
	}

	/*
	 * Check that the transport provider bound the
	 * address that we requested.
	 */
	if ((retp->addr.len != alen) ||
	  (memcmp(addr, retp->addr.buf, alen) != 0)) {
		log("Bound different address than requested\n");
		exit(1);
	}

	/*
	 * Free the memory for the t_bind structures we no
	 * longer need.
	 */
	t_free((char *)reqp, T_BIND);
	t_free((char *)retp, T_BIND);
	return(fd);
}
