/*
 * Copyright (c) 1993 Addison-Wesley Publishing Company.
 * All rights reserved.  This source code is provided
 * for its instructional value, and is not guaranteed
 * for any particular purpose.
 */
#include <rpc/rpc.h>
#include <netconfig.h>

SVCXPRT *
init_callback(const struct netconfig *ncp,
	void (*callback)(struct svc_req *, SVCXPRT *),
	ulong_t versnum, ulong_t *prognump)
{
	SVCXPRT *handle;
	int prognum;

	/*
	 * Create a server handle.
	 */
	handle = svc_tli_create(RPC_ANYFD, ncp, NULL, 0, 0);
	if (handle == NULL)
		return(NULL);

	/*
	 * Find an unused transient program number.
	 */
	for (prognum = 0x40000000; prognum < 0x60000000;
	  prognum++)
		if (rpcb_set(prognum, versnum, ncp,
		  &handle->xp_ltaddr))
			break;

	/*
	 * None found.  Get rid of the handle and return.
	 */
	if (prognum >= 0x60000000) {
		svc_destroy(handle);
		return(NULL);
	}

	/*
	 * Map the dispatching routine to the program and
	 * version numbers.
	 */
	if (!svc_reg(handle, prognum, versnum, callback,
	  NULL)) {
		svc_destroy(handle);
		rpcb_unset(prognum, versnum, ncp);
		return(NULL);
	}

	/*
	 * Return the program number and the handle.
	 */
	*prognump = prognum;
	return(handle);
}
